<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla02800Freqprat extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
	protected $parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'NRORG');
    protected $dataSourceManager;
    protected $util;
    protected $connection;
    protected $RestrContrService;

    protected $errors = array();

    public function __construct($crudService, $parameterBag, $dataSourceManager, $util, $connection, $RestrContrService) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->util              = $util;
        $this->connection        = $connection;
        $this->RestrContrService = $RestrContrService;
    }

    //Cascade save Vigencia->Servico->Prato
    public function saveVigencia($vigencia, $response, $dataSourceName = "vigfreqpratmes") {
        try {
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            unset($vigencia['__new_code_pk']);   
            $dataSet = new DataSet($dataSourceName, array($vigencia));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrvigencia = $rowsToSave[0]["NRVIGENCIA"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrvigencia;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveServico($servico, $response) {
        try {
            $dataSourceName = "freqpratmes";
            $servico["NRFREQPRATMES"] = $this->RestrContrService->getNewCode($dataSourceName);
            unset($servico['__new_code_pk']);  
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrfreqpratmes = $rowsToSave[0]["NRFREQPRATMES"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrfreqpratmes;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function savePrato(DTO\Request $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["vigencia"])) {
                $nrvigencia = $this->saveVigencia($row["vigencia"], $response);
            }
            if (isset($row["servico"])) {
                $row["servico"]["NRVIGENCIA"] = isset($nrvigencia) ? $nrvigencia : $row["servico"]["NRVIGENCIA"];
                $nrfreqpratmes = $this->saveServico($row["servico"], $response);
            }
            $row["prato"]["NRFREQPRATMES"] = isset($nrfreqpratmes) ? $nrfreqpratmes : $row["prato"]["NRFREQPRATMES"];
            $this->util->validateDuplicatedRecords($row["prato"], "itfreqpratmes", array("CDPRATO", "CDFILAUXPRAT", "NRFREQPRATMES"));
            $this->dataSourceName = "itfreqpratmes";
            $row["prato"]["NRITFREQPRATMES"] = $this->RestrContrService->getNewCode($this->dataSourceName);
            unset($row["prato"]['__new_code_pk']); 
            $dataSet = new DataSet($this->dataSourceName, array($row["prato"]));
            $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
            parent::save($newRequest, $response);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    //Importação - Exportação
    public function servicosImportExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $servicos = $row["NMSERVICO"];
            if (isset($row["vigencia"])) {
                $nrvigenciadest = $this->saveVigencia($row["vigencia"], $response);
            } else {
                $nrvigenciadest =  $row["NRVIGENCIA"];
            }
            $tpoper = $row["TPOPER"];

            if ($tpoper == "S") {
                $this->deleteFreqpratmesByNrvigencia($nrvigenciadest);
            }
            foreach ($servicos as $servico) {
                $pratos = $this->getItfreqpratmes($servico["NRFREQPRATMES"]);
                $nrfreqpratmesdest = $this->saveServicoImportExport($servico, $row, $nrvigenciadest);
                $pratosExisting = $this->getItfreqpratmes($nrfreqpratmesdest);
                $pratosExisting = array_map(function($prato){
                    return $prato["CDPRATO"].'|'.$prato["CDFILAUXPRAT"];
                }, $pratosExisting);
                $pratosToSave = array();
                foreach ($pratos as &$prato) {
                    if (in_array($prato["CDPRATO"].'|'.$prato["CDFILAUXPRAT"], $pratosExisting)) {
                        continue;
                    } else {
                        $prato["NRFREQPRATMES"] = $nrfreqpratmesdest;
                        $prato["NRITFREQPRATMES"] = $this->RestrContrService->getNewCode("ITFREQPRATMES");
                        $prato["__is_new"] = true;
                        array_push($pratosToSave, $prato);
                    }
                }
                $dataSourceName = "itfreqpratmes";
                $dataSet = new DataSet($dataSourceName, $pratosToSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function saveServicoImportExport($servico, $vigencia, $nrvigenciadest) {
        $dataSourceName = "freqpratmes";

        $conditions = $this->RestrContrService->buildConditions(array("CDSERVFILI", "NRVIGENCIA"), array($servico["CDSERVFILI"], $nrvigenciadest));
        $freqpratmes = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (!empty($freqpratmes)) {
            $nrfreqpratmes =  $freqpratmes[0]["NRFREQPRATMES"];
        } else {
            $servico["NRVIGENCIA"] = $nrvigenciadest;
            $servico["DTVIGENCIA"] = $vigencia["DTVIGENCIA"];
            $servico["CDFILIAL"] = $vigencia["CDFILIAL"];
            $servico["DSFREQPRATMES"] = $vigencia["DSFREQPRATMES"];
            $nrfreqpratmes = $this->RestrContrService->getNewCode("FREQPRATMES");
            $servico["NRFREQPRATMES"] = $nrfreqpratmes;
            $servico["__is_new"] = true;
            
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
        }
        return $nrfreqpratmes;
    }

    public function getItfreqpratmes($nrfreqpratmes) {
        $conditions = $this->RestrContrService->buildConditions(array("NRFREQPRATMES"), array($nrfreqpratmes));
        return $this->crudService->getFilteredDataSet("itfreqpratmes", $conditions)->getRows();
    }

    private function deleteFreqpratmesByNrvigencia($nrvigencia) {
        $sql = "DELETE FROM FREQPRATMES WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function vigenciaExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $vigenciaOrig = $row["VIGENCIAORIG"];
            $tables = $row["VIGENCIADEST"];
            foreach($tables as $table) {
                if (in_array(strtoupper($table), $this->RestrContrService->getTables())) { // Prevent SQL injection
                    $row = array(
                        "CDFILIAL" => $vigenciaOrig["CDFILIAL"],
                        "DTVIGENCIA" => $vigenciaOrig["DTVIGENCIA"],
                        "DTVIGENCIAFIM" => $vigenciaOrig["DTVIGENCIAFIM"],
                        "DESCRICAO" => $vigenciaOrig["DESCRICAO"],
                        "IDTIPORESTRICAO" => $vigenciaOrig["IDTIPORESTRICAO"],
                        "CDDESCOBSFILIAL" => $vigenciaOrig["CDDESCOBSFILIAL"],
                        "dataSourceName" => strtolower($table),
                        "fieldsToValidate" => array("CDFILIAL", "DTVIGENCIA","IDTIPORESTRICAO"),
                        "__is_new" => true
                    );
                    try {
                        $this->RestrContrService->validateRegisterVigencia($row);
                        if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                            $this->RestrContrService->validateIntervalOverlap(null, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
                        }
                        $this->saveVigencia($row, $response, $row["dataSourceName"]);
                    } catch( \Exception $e) {
                        if ($e->getMessage() == "registerExists") {
                            array_push($this->errors, $table);
                        } else {
                            throw $e;
                        }
                    }
                }
            }
            
            $this->connection->commit();
            if (!empty($this->errors)) {
                $response->addDataSet(new DataSet("errors", $this->errors));
                $response->setError(new Error("vigenciaExportErrors", 500));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function verificaParaExportacao($request, $response){
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->crudService->getEnvironment()->getNrOrg()
            ),
            array(
                "columnName" => "IDTIPORESTRICAO",
                "operator" => "=",
                "value" => $request->getRow()['IDTIPORESTRICAO']
            )
        );
        $dataSet = $this->crudService->getFilteredDataset("padgrupoprato", $conditions);
        $padraoPrato = $dataSet->getRows();
        
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->crudService->getEnvironment()->getNrOrg()
            )
        );
        
        $dataSet = $this->crudService->getFilteredDataset("servpadr", $conditions);
        $padraoServico = $dataSet->getRows();
        
        $response->addDataSet(new DataSet('RESULT', array(sizeof($padraoPrato) && sizeof($padraoServico))));
    }
}